from ..validators.sigla_estado_validator import SiglaEstadoValidator
from .base_field import Base

__all__ = ["SiglaEstado"]


class SiglaEstado(Base):
    """
    Sigla do Estado

    Exemplos: 'SP'
    """

    format = "sigla_estado"
    Validator = SiglaEstadoValidator
    examples = ["SP", "DF"]



from enum import Enum

try:
    import pydantic  # noqa
except ModuleNotFoundError:
    raise ModuleNotFoundError("Are you sure you installed Pydantic?")


class PydanticVersion(Enum):
    v1 = 1
    v2 = 2


def get_pydantic_version() -> PydanticVersion:
    if hasattr(pydantic, "__version__"):
        if pydantic.__version__.startswith("1"):
            return PydanticVersion.v1
        if pydantic.__version__.startswith("2"):
            return PydanticVersion.v2
    return PydanticVersion.v1



from ..validators.te_validator import TEValidator
from .base_field import BaseDigits

__all__ = ["TE"]


class TE(BaseDigits):
    """
    Título Eleitoral

    Exemplos: '000000000000'
    """

    format = "te"
    Validator = TEValidator
    examples = ["000000000000"]



from ..validators.renavam_validator import RENAVAMValidator
from .base_field import BaseDigits

__all__ = ["RENAVAM"]


class RENAVAM(BaseDigits):
    """
    Registro Nacional de Veículos Automotores

    Exemplos: '00000000000'
    """

    format = "renavam"
    Validator = RENAVAMValidator
    examples = ["00000000000"]



from ..tools.get_versions import get_pydantic_version

pydantic_version = get_pydantic_version()


if pydantic_version.value == 1:
    from .base_field_v1 import BaseDigitsv1 as BaseDigits  # type: ignore # noqa
    from .base_field_v1 import BaseMaskv1 as BaseMask  # type: ignore # noqa
    from .base_field_v1 import Basev1 as Base  # type: ignore # noqa

if pydantic_version.value == 2:
    from .base_field_v2 import BaseDigitsV2 as BaseDigits  # type: ignore # noqa
    from .base_field_v2 import BaseMaskV2 as BaseMask  # type: ignore # noqa
    from .base_field_v2 import BaseV2 as Base  # type: ignore # noqa



__version__ = "v1.1.0"

from .fields.base_field_errors import *
from .fields.cep_field import *
from .fields.certidao_field import *
from .fields.cnh_field import *
from .fields.cnpj_field import *
from .fields.cns_field import *
from .fields.cpf_field import *
from .fields.pis_field import *
from .fields.placa_veiculo_field import *
from .fields.renavam_field import *
from .fields.sigla_estado_field import *
from .fields.te_field import *



from ..validators.certidao_validator import CertidaoValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "Certidao",
    "CertidaoMask",
    "CertidaoDigits",
]


class Certidao(Base):
    """
    Certidão

    Exemplos: '00000000000000000000000000000000' ou '000000.00.00.0000.0.00000.000.0000000-00'
    """

    format = "certidao"
    Validator = CertidaoValidator
    mask = {"required": False, "format": "XXXXXX.XX.XX.XXXX.X.XXXXX.XXX.XXXXXXX-XX"}
    examples = [
        "00000000000000000000000000000000",
        "000000.00.00.0000.0.00000.000.0000000-00",
    ]


class CertidaoMask(BaseMask):
    """
    Certidão

    Exemplos: '000000.00.00.0000.0.00000.000.0000000-00'
    """

    format = "certidao"
    Validator = CertidaoValidator
    mask = {"required": True, "format": "XXXXXX.XX.XX.XXXX.X.XXXXX.XXX.XXXXXXX-XX"}
    examples = ["000000.00.00.0000.0.00000.000.0000000-00"]


class CertidaoDigits(BaseDigits):
    """
    Certidão

    Exemplos: '00000000000000000000000000000000'
    """

    format = "certidao"
    Validator = CertidaoValidator
    examples = ["00000000000000000000000000000000"]



from .base_validator import FieldValidator

__all__ = ["CNSValidator"]


class CNSValidator(FieldValidator):
    def __init__(self, cns: str) -> None:
        self.cns = str(cns)
        self.cns_digits = self._get_only_numbers(cns)

    def validate(self) -> bool:
        cns = self.cns_digits

        if len(cns) != 15 or len(set(cns)) == 1:
            return False

        if cns[0] not in ["1", "2", "7", "8", "9"]:
            return False

        if cns[0] in ["1", "2"]:
            return self._validate_first_case(cns)
        else:
            return self._validate_second_case(cns)

    def _validate_first_case(self, doc: str) -> bool:
        cns = [n for n in doc][:11]
        sum = 0
        for i in range(11):
            sum += int(cns[i]) * (15 - i)

        dv = 11 - (sum % 11)

        if dv == 11:
            dv = 0

        if dv == 10:
            sum += 2
            dv = 11 - (sum % 11)
            cns = cns + ["0", "0", "1", str(dv)]
        else:
            cns = cns + ["0", "0", "0", str(dv)]
        return "".join(cns) == doc

    def _validate_second_case(self, doc: str) -> bool:
        cns = [n for n in doc]
        sum = 0
        for i in range(15):
            sum += int(cns[i]) * (15 - i)
        return sum % 11 == 0



from typing import Any, Callable, Dict, Generator, List

from .base_field_errors import FieldTypes, raise_field

__all__ = ["BaseFieldClass"]


AnyCallable = Callable[..., Any]
CallableGenerator = Generator[AnyCallable, None, None]


class BaseFieldClass:
    format: str
    Validator: Callable[..., Any]
    mask: Dict = {"required": False, "format": None}
    examples: List[str] = []

    __slots__ = ["number"]

    @classmethod
    def validate_type(cls, value: Any) -> str:
        if not isinstance(value, str):
            raise_field(FieldTypes.type)
        return value

    @classmethod
    def validate(cls, value: str) -> str:
        doc = cls.Validator(value)
        if not doc.validate():
            raise_field(FieldTypes.invalid)
        return value

    @classmethod
    def validate_mask(cls, value: str) -> str:
        doc = cls.Validator(value)
        if not doc.validate_mask():
            raise_field(FieldTypes.mask)
        return value

    @classmethod
    def validate_numbers(cls, value: str) -> str:
        if not value.isdigit():
            raise_field(FieldTypes.digit)
        return value



from ..validators.pis_validator import PISValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "PIS",
    "PISMask",
    "PISDigits",
]


class PIS(Base):
    """
    Programa de Integração Social

    Exemplos: '00000000000' ou '000.00000.00-0'
    """

    format = "pis"
    Validator = PISValidator
    mask = {"required": False, "format": "XXX.XXXXX.XX-X"}
    examples = ["00000000000", "000.00000.00-0"]


class PISMask(BaseMask):
    """
    Programa de Integração Social

    Exemplos: '000.00000.00-0'
    """

    format = "pis mask"
    Validator = PISValidator
    mask = {"required": True, "format": "XXX.XXXXX.XX-X"}
    examples = ["000.00000.00-0"]


class PISDigits(BaseDigits):
    """
    Programa de Integração Social

    Exemplos: '00000000000'
    """

    format = "pis digits"
    Validator = PISValidator
    examples = ["00000000000"]



import re
from abc import ABC, abstractmethod


class FieldValidator(ABC):
    @abstractmethod
    def validate(self):
        ...

    def _get_only_numbers(self, value: str) -> str:
        return re.sub(r"\D", "", value)


class FieldMaskValidator(FieldValidator):
    @abstractmethod
    def validate_mask(self):
        ...



from ..validators.cpf_validator import CPFValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "CPF",
    "CPFMask",
    "CPFDigits",
]


class CPF(Base):
    """
    Cadastro de Pessoa Física

    Exemplos: '00000000000' ou '000.000.000-00'
    """

    format = "cpf"
    Validator = CPFValidator
    mask = {"required": False, "format": "000.000.000-00"}
    examples = ["00000000000", "000.000.000-00"]


class CPFMask(BaseMask):
    """
    Cadastro de Pessoa Física

    Exemplos: '000.000.000-00'
    """

    format = "cpf mask"
    Validator = CPFValidator
    mask = {"required": True, "format": "000.000.000-00"}
    examples = ["000.000.000-00"]


class CPFDigits(BaseDigits):
    """
    Cadastro de Pessoa Física

    Exemplos: '00000000000'
    """

    format = "cpf digits"
    Validator = CPFValidator
    examples = ["00000000000"]



from ..validators.cns_validator import CNSValidator
from .base_field import BaseDigits

__all__ = ["CNS"]


class CNS(BaseDigits):
    """
    Cartão Nacional de Saúde

    Exemplos: '000000000000000'
    """

    format = "cns"
    Validator = CNSValidator
    mask = {"required": True, "format": None}
    examples = ["000000000000000"]



from .base_validator import FieldValidator

__all__ = ["SiglaEstadoValidator"]


class SiglaEstadoValidator(FieldValidator):
    def __init__(self, sigla: str) -> None:
        self.sigla = str(sigla)

    def validate(self) -> bool:
        siglas = [
            "AC",
            "AL",
            "AM",
            "AM",
            "AP",
            "BA",
            "CE",
            "DF",
            "DF",
            "ES",
            "GO",
            "GO",
            "MA",
            "MG",
            "MS",
            "MT",
            "PA",
            "PB",
            "PE",
            "PI",
            "PR",
            "RJ",
            "RN",
            "RO",
            "RR",
            "RS",
            "SC",
            "SE",
            "SP",
            "TO",
        ]

        if len(self.sigla) != 2 or self.sigla.islower():
            return False
        return self.sigla in siglas



from ..validators.cnh_validator import CNHValidator
from .base_field import BaseDigits

__all__ = ["CNH"]


class CNH(BaseDigits):
    """
    Carteira Nacional de Habilitação

    Exemplos: '00000000000'
    """

    format = "cnh"
    Validator = CNHValidator
    examples = ["00000000000"]



from typing import Any, Callable, Dict, Generator

from ..validators.base_validator import FieldMaskValidator, FieldValidator
from .base_field_class import BaseFieldClass

__all__ = [
    "Basev1",
    "BaseMaskv1",
    "BaseDigitsv1",
]

AnyCallable = Callable[..., Any]
CallableGenerator = Generator[AnyCallable, None, None]


class Basev1(BaseFieldClass):
    Validator: Callable[..., FieldValidator]

    @classmethod
    def __modify_schema__(cls, field_schema: Dict[str, Any]) -> None:
        field_schema.update(
            type="string", format=cls.format, mask=cls.mask, example=cls.examples
        )

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate


class BaseMaskv1(Basev1):
    Validator: Callable[..., FieldMaskValidator]

    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate_mask
        yield cls.validate


class BaseDigitsv1(Basev1):
    @classmethod
    def __get_validators__(cls) -> CallableGenerator:
        yield cls.validate_type
        yield cls.validate_numbers
        yield cls.validate



from .base_validator import FieldValidator

__all__ = ["CNHValidator"]


class CNHValidator(FieldValidator):
    def __init__(self, cnh: str) -> None:
        self.cnh = str(cnh)
        self.cnh_digits = self._get_only_numbers(cnh)

    def validate(self) -> bool:
        cnh = self.cnh

        if len(self.cnh_digits) != 11 or len(set(self.cnh_digits)) == 1:
            return False

        v = 0
        j = 9

        for i in range(9):
            v += int(cnh[i]) * j
            j -= 1

        dsc = 0
        vl1 = v % 11

        if vl1 >= 10:
            vl1 = 0
            dsc = 2

        v = 0
        j = 1

        for i in range(9):
            v += int(cnh[i]) * j
            j += 1

        x = v % 11
        vl2 = 0 if x >= 10 else x - dsc

        return f"{vl1}{vl2}" == cnh[-2:]



from ..validators.cnpj_validator import CNPJValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "CNPJ",
    "CNPJMask",
    "CNPJDigits",
]


class CNPJ(Base):
    """
    Cadastro Nacional de Pessoas Jurídicas

    Exemplos: '00000000000000' ou '00.000.000/0000-00'
    """

    format = "cnpj"
    Validator = CNPJValidator
    mask = {"required": False, "format": "XX.XXX.XXX/XXXXX-XX"}
    examples = ["00000000000000", "00.000.000/0000-00"]


class CNPJMask(BaseMask):
    """
    Cadastro Nacional de Pessoas Jurídicas

    Exemplos: '00.000.000/0000-00'
    """

    format = "cnpj"
    Validator = CNPJValidator
    mask = {"required": True, "format": "XX.XXX.XXX/XXXXX-XX"}
    examples = ["00.000.000/0000-00"]


class CNPJDigits(BaseDigits):
    """
    Cadastro Nacional de Pessoas Jurídicas

    Exemplos: '00000000000000'
    """

    format = "cnpj"
    Validator = CNPJValidator
    examples = ["00000000000000"]



import re

from .base_validator import FieldValidator

__all__ = ["PlacaVeiculoValidator"]


class PlacaVeiculoValidator(FieldValidator):
    def __init__(self, placa: str) -> None:
        self.placa = str(placa)

    def validate(self) -> bool:
        placa = self.placa
        regex_mercosul = r"^[A-Z]{3}[0-9][A-Z][0-9]{2}$"
        regex_old = r"^[A-Z]{3}[0-9]{4}$"

        if bool(re.match(regex_mercosul, placa)) or bool(re.match(regex_old, placa)):
            return True
        return False



from .base_validator import FieldMaskValidator

__all__ = ["CNPJValidator"]


class CNPJValidator(FieldMaskValidator):
    def __init__(self, cnpj) -> None:
        self.cnpj = str(cnpj)
        self.cnpj_digits = self._get_only_numbers(cnpj)

    def validate_mask(self) -> bool:
        if len(self.cnpj) != 18 or len(self.cnpj_digits) != 14:
            return False

        if (
            self.cnpj[2:3] == "."
            and self.cnpj[6:7] == "."
            and self.cnpj[10:11] == "/"
            and self.cnpj[15:16] == "-"
        ):
            return True
        return False

    def validate(self) -> bool:
        if len(self.cnpj_digits) != 14:
            return False

        first_digit = self._validate_first_digit()
        second_digit = self._validate_second_digit()
        return (
            self.cnpj_digits[12] == first_digit and self.cnpj_digits[13] == second_digit
        )

    def _validate_first_digit(self) -> str:
        sum = 0
        weight = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]

        for n in range(12):
            value = int(self.cnpj_digits[n]) * weight[n]
            sum = sum + value

        check_digit = sum % 11

        if check_digit < 2:
            first_digit = 0
        else:
            first_digit = 11 - check_digit
        return str(first_digit)

    def _validate_second_digit(self) -> str:
        sum = 0
        weight = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
        for n in range(13):
            sum = sum + int(self.cnpj_digits[n]) * weight[n]

        check_digit = sum % 11

        if check_digit < 2:
            second_digit = 0
        else:
            second_digit = 11 - check_digit
        return str(second_digit)



from typing import List

from .base_validator import FieldMaskValidator

__all__ = ["PISValidator"]


class PISValidator(FieldMaskValidator):
    def __init__(self, pis: str) -> None:
        self.pis = str(pis)
        self.pis_digits = self._get_only_numbers(pis)

    def validate_mask(self) -> bool:
        if len(self.pis) != 14 or len(self.pis_digits) != 11:
            return False

        if self.pis[3:4] == "." and self.pis[9:10] == "." and self.pis[12:13] == "-":
            return True
        return False

    def validate(self) -> bool:
        pis = [int(n) for n in list(self.pis_digits)]

        if len(set(pis)) == 1:
            return False

        if len(pis) != 11:
            return False

        return self._validate_digit(pis)

    def _validate_digit(self, pis: List) -> bool:
        multipliers = [3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
        summation = 0

        for position in range(0, 10):
            summation += int(pis[position]) * multipliers[position]

        module = summation % 11
        dv = 11 - module

        if dv in [10, 11]:
            dv = 0

        return dv == pis[10]



from ..validators.placa_veiculo_validator import PlacaVeiculoValidator
from .base_field import Base

__all__ = ["PlacaVeiculo"]


class PlacaVeiculo(Base):
    """
    Placa veicular

    Exemplos: 'ABC0000' ou 'ABC0D00'
    """

    format = "placa_veiculo"
    Validator = PlacaVeiculoValidator
    examples = ["ABC0000", "ABC0D00"]



from .base_validator import FieldValidator

__all__ = ["RENAVAMValidator"]


class RENAVAMValidator(FieldValidator):
    def __init__(self, reavam: str) -> None:
        self.renavam = str(reavam)
        self.renavam_digits = self._get_only_numbers(reavam)

    def validate(self) -> bool:
        renavam = self.renavam_digits

        if len(renavam) == 9:
            renavam = "00{}".format(renavam)

        if len(renavam) != 11 or len(set(renavam)) == 1:
            return False

        renavam_with_no_digit = renavam[:-1]
        renavam_with_no_digit = renavam_with_no_digit[::-1]

        sum = 0
        for i, digito in enumerate(renavam_with_no_digit[:8]):
            sum += int(digito) * (i + 2)

        sum += int(renavam_with_no_digit[8]) * 2
        sum += int(renavam_with_no_digit[9]) * 3

        mod11 = sum % 11

        last_computed_number = 11 - mod11

        if last_computed_number >= 10:
            last_computed_number = 0

        return last_computed_number == int(renavam[-1])



from ..validators.cep_validator import CEPValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "CEP",
    "CEPMask",
    "CEPDigits",
]


class CEP(Base):
    """
    Código de Endereçamento Postal

    Exemplos: '00000000' ou '00000-000'
    """

    format = "cep"
    Validator = CEPValidator
    mask = {"required": False, "format": "XXXXX-XXX"}
    examples = ["00000000", "00000-000"]


class CEPMask(BaseMask):
    """
    Código de Endereçamento Postal

    Exemplos: '00000-000'
    """

    format = "cep"
    Validator = CEPValidator
    mask = {"required": True, "format": "XXXXX-XXX"}
    examples = ["00000-000"]


class CEPDigits(BaseDigits):
    """
    Código de Endereçamento Postal

    Exemplos: '00000000'
    """

    format = "cep"
    Validator = CEPValidator
    examples = ["00000000"]



from __future__ import annotations

from typing import Any, Dict, Mapping

from .base_field_class import BaseFieldClass

try:
    from pydantic_core import core_schema  # type: ignore
except ModuleNotFoundError:
    raise ModuleNotFoundError("Are you sure you installed pydantic_core")

CoreSchema: Any = Mapping[str, Any]
JsonSchemaValue = Dict[str, Any]


class BaseV2(BaseFieldClass):
    @classmethod
    def __get_pydantic_core_schema__(
        cls,
        source,
        *args,
        **kwargs,
    ) -> CoreSchema:
        try:
            schema = core_schema.with_info_after_validator_function(
                cls._validate, core_schema.str_schema()
            )
        except Exception:
            schema = core_schema.general_after_validator_function(
                cls._validate, core_schema.str_schema()
            )
        return schema

    @classmethod
    def __get_pydantic_json_schema__(
        cls,
        core_schema: CoreSchema,
        handler,
        *args,
        **kwargs,
    ) -> JsonSchemaValue:
        field_schema = handler(core_schema)
        field_schema.update(
            type="string", format=cls.format, mask=cls.mask, example=cls.examples
        )
        return field_schema

    @classmethod
    def _validate(cls, __input_value: str, *args, **kwargs) -> str:
        cls.validate_type(__input_value)
        return cls.validate(__input_value)


class BaseMaskV2(BaseV2):
    @classmethod
    def _validate(cls, __input_value: str, *args, **kwargs) -> str:
        cls.validate_type(__input_value)
        cls.validate_mask(__input_value)
        return cls.validate(__input_value)


class BaseDigitsV2(BaseV2):
    @classmethod
    def _validate(cls, __input_value: str, *args, **kwargs) -> str:
        cls.validate_type(__input_value)
        cls.validate_numbers(__input_value)
        return cls.validate(__input_value)
